@@ -4,11 +4,12 @@ use strict;
 use warnings;
 use 5.010;
 
-our $VERSION = '1.06';
+our $VERSION = '1.07';
 
 use Getopt::Long;
 use Net::MPD;
 use Proc::Daemon;
+use Log::Dispatch;
 
 sub new {
   my ($class, @options) = @_;
@@ -24,7 +25,6 @@ sub new {
     mpd        => undef,
     mpd_conn   => 'localhost',
     music_path => 'music',
-    verbose    => 0,
     @options
   }, $class;
 }
@@ -35,12 +35,6 @@ sub mpd {
   $self->{mpd};
 }
 
-sub say {
-  my ($self, @args) = @_;
-
-  say @args if $self->{verbose};
-}
-
 sub parse_options {
   my ($self, @options) = @_;
 
@@ -57,7 +51,8 @@ sub parse_options {
     'h|help'         => sub { $self->{action} = 'show_help' },
     'mpd=s'          => \$self->{mpd_conn},
     'music-path=s'   => \$self->{music_path},
-    'v|verbose!'     => \$self->{verbose},
+    's|syslog=s'     => \$self->{syslog},
+    'l|conlog=s'     => \$self->{conlog},
   );
 }
 
@@ -74,8 +69,18 @@ sub execute {
     $self->$action() and return 1;
   }
 
+  @SIG{qw( INT TERM HUP )} = sub { $self->safe_exit() };
+
+  my @loggers;
+  push @loggers, ([ 'Screen', min_level => $self->{conlog}, newline => 1 ])
+    if $self->{conlog};
+  push @loggers, ([ 'Syslog', min_level => $self->{syslog}, ident => 'mpdj' ])
+    if $self->{syslog};
+
+  $self->{log} = Log::Dispatch->new(outputs => \@loggers);
+
   if ($self->{daemon}) {
-    $self->say('Forking to background');
+    $self->{log}->notice('Forking to background');
     Proc::Daemon::Init;
   }
 
@@ -87,8 +92,9 @@ sub execute {
   $self->update_cache;
 
   while (1) {
-    $self->say('Waiting');
-    my @changes = $self->mpd->idle(qw(database player playlist message options));
+    $self->{log}->debug('Waiting');
+    my @changes =
+      $self->mpd->idle(qw(database player playlist message options));
     $self->mpd->update_status();
 
     foreach my $subsystem (@changes) {
@@ -101,7 +107,7 @@ sub execute {
 sub configure {
   my ($self) = @_;
 
-  $self->say('Configuring MPD server');
+  $self->{log}->notice('Configuring MPD server');
 
   $self->mpd->repeat(0);
   $self->mpd->random(0);
@@ -109,24 +115,27 @@ sub configure {
   if ($self->{calls_freq}) {
     my $now = time;
     $self->{last_call} = $now - $now % $self->{calls_freq};
-    $self->say("Set last call to $self->{last_call}");
+    $self->{log}->notice("Set last call to $self->{last_call}");
   }
 }
 
 sub update_cache {
   my ($self) = @_;
 
-  $self->say('Updating music and calls cache...');
+  $self->{log}->notice('Updating music and calls cache...');
 
-  foreach my $category ( ('music', 'calls') ) {
+  foreach my $category (('music', 'calls')) {
 
-    @{$self->{$category}} = grep { $_->{type} eq 'file' } $self->mpd->list_all($self->{"${category}_path"});
+    @{ $self->{$category} } = grep { $_->{type} eq 'file' }
+      $self->mpd->list_all($self->{"${category}_path"});
 
-    my $total = scalar(@{$self->{$category}});
+    my $total = scalar(@{ $self->{$category} });
     if ($total) {
-      $self->say(sprintf("Total %s available: %d", $category, $total));
+      $self->{log}
+        ->notice(sprintf("Total %s available: %d", $category, $total));
     } else {
-      $self->say("No $category available.  Path is mpd path not file system.");
+      $self->{log}
+        ->warning("No $category available.  Path is mpd path not file system.");
     }
   }
 }
@@ -137,7 +146,7 @@ sub remove_old_songs {
   my $song = $self->mpd->song || 0;
   my $count = $song - $self->{before};
   if ($count > 0) {
-    $self->say("Deleting $count old songs");
+    $self->{log}->info("Deleting $count old songs");
     $self->mpd->delete("0:$count");
   }
 }
@@ -148,7 +157,7 @@ sub add_new_songs {
   my $song = $self->mpd->song || 0;
   my $count = $self->{after} + $song - $self->mpd->playlist_length + 1;
   if ($count > 0) {
-    $self->say("Adding $count new songs");
+    $self->{log}->info("Adding $count new songs");
     $self->add_song for 1 .. $count;
   }
 }
@@ -162,27 +171,27 @@ sub add_song {
 sub add_call {
   my ($self) = @_;
 
-  $self->say('Injecting call');
+  $self->{log}->info('Injecting call');
 
   $self->add_random_item_from_category('calls', 'immediate');
 
   my $now = time;
   $self->{last_call} = $now - $now % $self->{calls_freq};
-  $self->say('Set last call to ' . $self->{last_call});
+  $self->{log}->info('Set last call to ' . $self->{last_call});
 }
 
 sub add_random_item_from_category {
   my ($self, $category, $next) = @_;
 
-  my @items = @{$self->{$category}};
+  my @items = @{ $self->{$category} };
 
   my $index = int(rand(scalar @items));
-  my $item = $items[$index];
+  my $item  = $items[$index];
 
-  my $uri = $item->{uri};
+  my $uri  = $item->{uri};
   my $song = $self->mpd->song || 0;
   my $pos  = $next ? $song + 1 : $self->mpd->playlist_length;
-  $self->say('Adding ' . $uri . ' at position ' . $pos);
+  $self->{log}->info('Adding ' . $uri . ' at position ' . $pos);
 
   $self->mpd->add_id($uri, $pos);
 }
@@ -200,6 +209,12 @@ sub show_version {
   say "mpdj (App::MPDJ) version $VERSION";
 }
 
+sub safe_exit {
+  my ($self) = @_;
+
+  $self->{log}->log_and_die(level => 'notice', message => 'Ending');
+}
+
 sub show_help {
   my ($self) = @_;
 
@@ -208,7 +223,8 @@ Usage: mpdj [options]
 
 Options:
   --mpd             MPD connection string (password\@host:port)
-  -v,--verbose      Turn on chatty output
+  -s --syslog       Turns on syslog output (debug, info, notice, warn[ing], error, etc)
+  -l,--conlog       Turns on console output (same choices as --syslog)
   --no-daemon       Turn off daemonizing
   -b,--before       Number of songs to keep in playlist before current song
   -a,--after        Number of songs to keep in playlist after current song
@@ -254,7 +270,7 @@ sub message_changed {
 sub options_changed {
   my $self = shift;
 
-  $self->say('Resetting configuration');
+  $self->{log}->notice('Resetting configuration');
 
   $self->mpd->repeat(0);
   $self->mpd->random(0);
@@ -267,7 +283,7 @@ sub handle_message_mpdj {
 
   if ($option =~ /^(?:before|after|calls_freq)$/) {
     return unless $value =~ /^\d+$/;
-    $self->say('Setting ' . $option . ' to ' . $value);
+    $self->{log}->info('Setting ' . $option . ' to ' . $value);
     $self->{$option} = $value;
     $self->player_changed();
   }
@@ -303,9 +319,17 @@ of random songs for you just like a real DJ.
 Sets the MPD connection details.  Should be a string like password@host:port.
 The password and port are both optional.
 
-=item -v, --verbose
+=item -s, --syslog
 
-Makes the output verbose.  Default is to be quiet.
+Turns on sending of log information to syslog at specified level.  Level is a
+required parameter can be one of debug, info, notice, warn[ing], err[or],
+crit[ical], alert or emerg[ency].
+
+=item -l, --conlog
+
+Turns on sending of log information to console at specified level.  Level is a
+required parameter can be one of debug, info, notice, warn[ing], err[or],
+crit[ical], alert or emerg[ency].
 
 =item --no-daemon
 
